#!/usr/bin/perl -w
# A version of the dreamscape program written in perl. 
# Jeremy English <jhe@jeremyenglish.org> 20-October-2005
#
#  This program reads in  5 list, adjectives, nouns, verbs,
#  prepositional phrases and adverbs. It then reads in a schema file
#  that tells the program the structure of the output. The
#  program fills in the variables, based on the type, from random data
#  pulled form the list.
#
#  The Data types of the schema files are:
#
#    adj  for adjectives
#    noun for nouns
#    verb for verbs
#    prep for prepositional phrase
#    adv  for adverbs
#
#  A caret in front of the datatype make the first letter of the word
#  upper case. ex: 
#    (^adv av1) the (noun n1) (verb v2).
#
#  The program searches the schema for tags in parenthesis. Data in
#  parenthesis tells the program the data type and the variable name. If
#  the variable name is in the lookup table then the value stored is
#  used. Otherwise, random data is assigned to the variable, inserted
#  in the position, and stored in the lookup table.

my $progname = "dreamscape";
my $adj_fn   = "adjective.txt";
my $nouns_fn = "nouns.txt";
my $verbs_fn = "verbs.txt";
my $prep_fn  = "prepositions.txt";
my $adv_fn   = "adverbs.txt";
my $scm_fn   = "schema.txt";
my @adj_ln   = ();
my @nouns_ln = ();
my @verbs_ln = ();
my @prep_ln  = ();
my @adv_ln   = ();
my @scm_ln   = ();
my %lookup   = ();

sub file_to_list{
  my ($fn) = @_;
  my @L = ();
  local *IN;
  open (IN, $fn) or die $!;
  while (<IN>) {
    push (@L, $_);
  }
  close (IN);
  return @L;
}

sub random_item{
  return $_[rand($#_)];
}

sub get_variable{
  my ($type, $var) = @_;
  my $val;
  my $upcase = 0;
  $type =~ s/( |\r|\t|\n)//g;
  if ($type =~ /\^.+/){
    $upcase = 1;
    $type =~ s/\^//;
  }
  if ($lookup{$var}){
    $val = $lookup{$var};
  }
  else{
    if ($type eq "adj"){
      $val = random_item(@adj_ln);
    }
    elsif ($type eq "noun"){
      $val = random_item(@nouns_ln);
    }
    elsif ($type eq "verb"){
      $val = random_item(@verbs_ln);
    }
    elsif ($type eq "prep"){
      $val = random_item(@prep_ln);
    }
    elsif ($type eq "adv"){
      $val = random_item(@adv_ln);
    }
    else{
      die "Invalid type $type";
    }
    $lookup{$var} = $val;
  }
  $val =~ s/(\r|\n)//g;
  if ($upcase) {
    $val = ucfirst $val;
  }
  return $val;

}

sub main{

  my $regular        = 0;
  my $read_variable  = 1;
  my $read_data_type = 2;
  my $state          = $regular;
  my $var            = "";
  my $type           = "";
  my $output         = "";

  @adj_ln         = file_to_list($adj_fn);
  @nouns_ln       = file_to_list($nouns_fn);
  @verbs_ln       = file_to_list($verbs_fn);
  @prep_ln        = file_to_list($prep_fn);
  @adv_ln         = file_to_list($adv_fn);
  @scm_ln         = file_to_list($scm_fn);

  srand();
  for my $elem (@scm_ln){
    my @tokens = split(/(\(|\)|  *)/, $elem);
    for my $token (@tokens){
      if ($token =~ m/[^  *]/){
	if ($state == $read_variable){
	  $var = $token;
	  $state = $regular;
	  $output .= get_variable($type, $var);
	  $output .= " ";
	}
	elsif ($state == $read_data_type){
	  $type = $token;
	  $state = $read_variable;
	}
	elsif ($token eq "("){
	  $state = $read_data_type;
	}
	elsif ($token eq ")"){
	  $state = $regular;
	}
	else{
	  $token =~ s/(\r|\n)//g;
	  chomp($token);
	  $output .= "$token ";
	}
      }
    }
    $output .= "\n";
  }
  $output =~ s/  *\././g;
  print $output;
  return 0;
}

main;
